@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -270,7 +270,7 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
     const float *decode = pim->Decode;
     gs_matrix_double mat;
     int index_bps;
-    const gs_color_space *pcs = pim->ColorSpace;
+    gs_color_space *pcs = pim->ColorSpace;
     gs_logical_operation_t lop = (pgs ? pgs->log_op : lop_default);
     int code;
     int log2_xbytes = (bps <= 8 ? 0 : arch_log2_sizeof_frac);
@@ -432,32 +432,70 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
     }
 
     /* Can we restrict the amount of image we need? */
-    while (pcpath) /* So we can break out of it */
+    while (!pim->imagematrices_are_untrustworthy) /* So we can break out of it */
     {
-        gs_rect rect, rect_out;
+        gs_rect rect, rect_src;
         gs_matrix mi;
         const gs_matrix *m = pgs != NULL ? &ctm_only(pgs) : NULL;
-        gs_fixed_rect obox;
         gs_int_rect irect;
         if (m == NULL || (code = gs_matrix_invert(m, &mi)) < 0 ||
             (code = gs_matrix_multiply(&mi, &pic->ImageMatrix, &mi)) < 0) {
             /* Give up trying to shrink the render box, but continue processing */
             break;
         }
-        gx_cpath_outer_box(pcpath, &obox);
-        rect.p.x = fixed2float(obox.p.x);
-        rect.p.y = fixed2float(obox.p.y);
-        rect.q.x = fixed2float(obox.q.x);
-        rect.q.y = fixed2float(obox.q.y);
-        code = gs_bbox_transform(&rect, &mi, &rect_out);
+        if (pcpath)
+        {
+            gs_fixed_rect obox;
+            gx_cpath_outer_box(pcpath, &obox);
+            rect.p.x = fixed2float(obox.p.x);
+            rect.p.y = fixed2float(obox.p.y);
+            rect.q.x = fixed2float(obox.q.x);
+            rect.q.y = fixed2float(obox.q.y);
+        }
+        else
+        {
+            rect.p.x = 0;
+            rect.p.y = 0;
+            rect.q.x = dev->width;
+            rect.q.y = dev->height;
+        }
+        /* rect is in destination space. Calculate rect_src, in source space. */
+        code = gs_bbox_transform(&rect, &mi, &rect_src);
         if (code < 0) {
             /* Give up trying to shrink the render/decode boxes, but continue processing */
             break;
         }
-        irect.p.x = (int)(rect_out.p.x-1.0);
-        irect.p.y = (int)(rect_out.p.y-1.0);
-        irect.q.x = (int)(rect_out.q.x+1.0);
-        irect.q.y = (int)(rect_out.q.y+1.0);
+        /* Need to expand the region to allow for the fact that the mitchell
+         * scaler reads multiple pixels in. */
+        /* If mi.{xx,yy} > 1 then we are downscaling. During downscaling,
+         * the support increases to ensure that we don't lose pixels contributions
+         * entirely. */
+        if (pim->Interpolate)
+        {
+            float support = any_abs(mi.xx);
+            int isupport;
+            if (any_abs(mi.yy) > support)
+                support = any_abs(mi.yy);
+            if (any_abs(mi.xy) > support)
+                support = any_abs(mi.xy);
+            if (any_abs(mi.yx) > support)
+                support = any_abs(mi.yx);
+            /* If upscaling (support < 1) then we need 2 extra lines on each side of the source region
+             * (2 being the maximum support for mitchell scaling).
+             * If downscaling, then the number of lines is increased to avoid individual
+             * contributions dropping out. */
+            isupport = 2; /* Mitchell support. */
+            if (support > 1)
+                isupport = (int)ceil(isupport * support);
+            rect_src.p.x -= isupport;
+            rect_src.p.y -= isupport;
+            rect_src.q.x += isupport;
+            rect_src.q.y += isupport+1; /* +1 is a fudge! */
+        }
+        irect.p.x = (int)floor(rect_src.p.x);
+        irect.p.y = (int)floor(rect_src.p.y);
+        irect.q.x = (int)ceil(rect_src.q.x);
+        irect.q.y = (int)ceil(rect_src.q.y);
         /* We therefore only need to render within irect. Restrict rrect to this. */
         if (penum->rrect.x < irect.p.x) {
             penum->rrect.w -= irect.p.x - penum->rrect.x;
@@ -481,29 +519,6 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
             if (penum->rrect.h < 0)
                 penum->rrect.h = 0;
         }
-        /* Need to expand the region to allow for the fact that the mitchell
-         * scaler reads multiple pixels in. */
-        /* If mi.{xx,yy} > 1 then we are downscaling. During downscaling,
-         * the support increases to ensure that we don't lose pixels contributions
-         * entirely. */
-        /* I do not understand the need for the +/- 1 fudge factors,
-         * but they seem to be required. Increasing the decode rectangle can
-         * never be bad at least... RJW */
-        {
-            float support = any_abs(mi.xx);
-            int isupport;
-            if (any_abs(mi.yy) > support)
-                support = any_abs(mi.yy);
-            if (any_abs(mi.xy) > support)
-                support = any_abs(mi.xy);
-            if (any_abs(mi.yx) > support)
-                support = any_abs(mi.yx);
-            isupport = (int)(MAX_ISCALE_SUPPORT * (support+1)) + 1;
-            irect.p.x -= isupport;
-            irect.p.y -= isupport;
-            irect.q.x += isupport;
-            irect.q.y += isupport;
-        }
         if (penum->drect.x < irect.p.x) {
             penum->drect.w -= irect.p.x - penum->drect.x;
             if (penum->drect.w < 0)
@@ -630,6 +645,9 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
         penum->icolor0 = &(penum->icolor0_val);
         penum->icolor1 = &(penum->icolor1_val);
     }
+    penum->icolor0->tag = (dev->graphics_type_tag & ~GS_DEVICE_ENCODES_TAGS);
+    penum->icolor1->tag = (dev->graphics_type_tag & ~GS_DEVICE_ENCODES_TAGS);
+
     if (masked) {       /* This is imagemask. */
         if (bps != 1 || pcs != NULL || penum->alpha || decode[0] == decode[1]) {
             code = gs_error_rangecheck;
@@ -806,6 +824,7 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
          image_skewed);
     penum->pgs = pgs;
     penum->pcs = pcs;
+    rc_increment_cs(pcs); /* Grab a ref (will decrement in gx_image1_end_image() */
     penum->memory = mem;
     penum->buffer = buffer;
     penum->buffer_size = bsize;
@@ -1100,11 +1119,11 @@ decode_range_needed(gx_image_enum *penum)
     bool scale = true;
 
     if (penum->map[0].decoding == sd_compute) {
-        if (!(gs_color_space_is_ICC(penum->pcs) || 
+        if (!(gs_color_space_is_ICC(penum->pcs) ||
             gs_color_space_is_PSCIE(penum->pcs))) {
             scale = false;
-        } 
-    } 
+        }
+    }
     return scale;
 }
 
@@ -1133,6 +1152,7 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
     gsicc_bufferdesc_t input_buff_desc;
     gsicc_bufferdesc_t output_buff_desc;
     gx_color_value conc[GX_DEVICE_COLOR_MAX_COMPONENTS];
+    int code;
 
     if (penum->icc_link == NULL) {
         return gs_rethrow(-1, "ICC Link not created during image render color");
@@ -1265,10 +1285,13 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
         gsicc_init_buffer(&output_buff_desc, num_des_comp, 1, false, false, false,
                           0, num_entries * num_des_comp,
                       1, num_entries);
-        (penum->icc_link->procs.map_buffer)(penum->dev, penum->icc_link, 
-                                            &input_buff_desc, &output_buff_desc, 
+        code = (penum->icc_link->procs.map_buffer)(penum->dev, penum->icc_link,
+                                            &input_buff_desc, &output_buff_desc,
                                             (void*) temp_buffer,
                                             (void*) penum->color_cache->device_contone);
+        if (code < 0)
+            return gs_rethrow(code, "Failure to map color buffer");
+
         /* Check if we need to apply any transfer functions.  If so then do it now */
         if (has_transfer) {
             for (k = 0; k < num_entries; k++) {
@@ -1439,12 +1462,12 @@ image_init_colors(gx_image_enum * penum, int bps, int spp,
             cc.paint.values[0] = real_decode[0];
             code = (*pcs->type->remap_color) (&cc, pcs, penum->icolor0,
                                        pgs, dev, gs_color_select_source);
-            if (code < 0) 
+            if (code < 0)
                 return code;
             cc.paint.values[0] = real_decode[1];
             code = (*pcs->type->remap_color) (&cc, pcs, penum->icolor1,
                                        pgs, dev, gs_color_select_source);
-            if (code < 0) 
+            if (code < 0)
                 return code;
         }
     }
