@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -348,8 +348,10 @@ pdf_begin_write_image(gx_device_pdf * pdev, pdf_image_writer * piw,
     }
     pdev->strm = pdev->streams.strm;
     pdev->strm = cos_write_stream_alloc(data, pdev, "pdf_begin_write_image");
-    if (pdev->strm == 0)
+    if (pdev->strm == 0) {
+        pdev->strm = save_strm;
         return_error(gs_error_VMerror);
+    }
     if (!mask)
         piw->data = data;
     piw->height = h;
@@ -412,6 +414,9 @@ pdf_begin_image_data(gx_device_pdf * pdev, pdf_image_writer * piw,
     if (pdev->JPEG_PassThrough) {
         CHECK(cos_dict_put_c_strings(pcd, "/Filter", "/DCTDecode"));
     }
+    if (pdev->JPX_PassThrough) {
+        CHECK(cos_dict_put_c_strings(pcd, "/Filter", "/JPXDecode"));
+    }
     return code;
 }
 
@@ -519,7 +524,7 @@ smask_image_check(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *p
                 if (p > v->contents.chars.data + v->contents.chars.size)
                     return 0;
                 ix *= 10;
-                ix += (*p) - 0x30;
+                ix += (*p++) - 0x30;
             }
             if (ix != pdev->image_mask_id)
                 return 0;
