@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2022 Artifex Software, Inc.
+/* Copyright (C) 2019-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -66,7 +66,7 @@
  * stores all the objects (which we don't want to do because its wasteful) and checking
  * to see if its already tested a given resource for spots/transparency.
  * This is a temporary allocation, big enough to hold all the objects in the file (1 per bit)
- * each time we have fully checked a resource we add it here, when checking a resoruce we
+ * each time we have fully checked a resource we add it here, when checking a resource we
  * first check this list to see if its already been checked, in which case we can skip
  * it. When done we release the memory.
  */
@@ -74,6 +74,7 @@ typedef struct {
     bool transparent;
     bool has_overprint; /* Does it have OP or op in an ExtGState? */
     pdf_dict *spot_dict;
+    pdf_array *font_array;
     uint32_t size;
     byte *CheckedResources;
 } pdfi_check_tracker_t;
@@ -84,6 +85,7 @@ static inline bool resource_is_checked(pdfi_check_tracker_t *tracker, pdf_obj *o
 {
     uint32_t byte_offset;
     byte bit_offset;
+    int object_num;
 
     if(tracker->CheckedResources == NULL)
         return 0;
@@ -91,14 +93,15 @@ static inline bool resource_is_checked(pdfi_check_tracker_t *tracker, pdf_obj *o
     /* objects with object number 0 are directly defined, we can't
      * store those so just return immediately
      */
-    if (o->object_num > 0 && (o->object_num >> 3) < tracker->size) {
+    object_num = pdf_object_num(o);
+    if (object_num > 0 && (object_num >> 3) < tracker->size) {
         /* CheckedResources is a byte array, each byte represents
          * 8 objects. So the object number / 8 is the byte offset
          * into the array, and then object number % 8 is the bit
          * within that byte that we want.
          */
-        bit_offset = 0x01 << (o->object_num % 8);
-        byte_offset = o->object_num >> 3;
+        bit_offset = 0x01 << (object_num % 8);
+        byte_offset = object_num >> 3;
 
         /* If its already set, then return that. */
         if (tracker->CheckedResources[byte_offset] & bit_offset)
@@ -116,12 +119,13 @@ pdfi_check_free_tracker(pdf_context *ctx, pdfi_check_tracker_t *tracker)
 {
     gs_free_object(ctx->memory, tracker->CheckedResources, "pdfi_check_free_tracker(flags)");
     pdfi_countdown(tracker->spot_dict);
+    pdfi_countdown(tracker->font_array);
     memset(tracker, 0, sizeof(*tracker));
     return 0;
 }
 
 static int
-pdfi_check_init_tracker(pdf_context *ctx, pdfi_check_tracker_t *tracker)
+pdfi_check_init_tracker(pdf_context *ctx, pdfi_check_tracker_t *tracker, pdf_array **fonts_array, pdf_array **spot_array)
 {
     int code = 0;
 
@@ -136,16 +140,25 @@ pdfi_check_init_tracker(pdf_context *ctx, pdfi_check_tracker_t *tracker)
     memset(tracker->CheckedResources, 0x00, tracker->size);
 
     if (ctx->device_state.spot_capable ||
-        (ctx->pgs->device->icc_struct->overprint_control) == gs_overprint_control_simulate) {
+        (ctx->pgs->device->icc_struct->overprint_control) == gs_overprint_control_simulate ||
+        spot_array != NULL)
+    {
         code = pdfi_dict_alloc(ctx, 32, &tracker->spot_dict);
         if (code < 0)
             goto cleanup;
         pdfi_countup(tracker->spot_dict);
     }
 
+    if (fonts_array != NULL) {
+        code = pdfi_array_alloc(ctx, 0, &tracker->font_array);
+        if (code < 0)
+            goto cleanup;
+        pdfi_countup(tracker->font_array);
+    }
+
     return 0;
 
- cleanup:
+cleanup:
     pdfi_check_free_tracker(ctx, tracker);
     return code;
 }
@@ -164,6 +177,9 @@ static int pdfi_check_ColorSpace_dict(pdf_context *ctx, pdf_dict *cspace_dict,
     if (resource_is_checked(tracker, (pdf_obj *)cspace_dict))
         return 0;
 
+    if (pdfi_type_of(cspace_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     if (pdfi_dict_entries(cspace_dict) > 0) {
         code = pdfi_loop_detector_mark(ctx); /* Mark the start of the ColorSpace dictionary loop */
         if (code < 0)
@@ -197,7 +213,7 @@ static int pdfi_check_ColorSpace_dict(pdf_context *ctx, pdf_dict *cspace_dict,
                 }
 
                 code = pdfi_dict_next(ctx, cspace_dict, &Key, &Value, &index);
-                if (code == 0 && Value->type == PDF_ARRAY)
+                if (code == 0 && pdfi_type_of(Value) == PDF_ARRAY)
                     break;
                 pdfi_countdown(Key);
                 Key = NULL;
@@ -235,6 +251,9 @@ static int pdfi_check_Shading(pdf_context *ctx, pdf_obj *shading,
     if (code < 0)
         return code;
 
+    if (pdfi_type_of(shading_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     code = pdfi_dict_knownget(ctx, shading_dict, "ColorSpace", (pdf_obj **)&o);
     if (code > 0) {
         code = pdfi_check_ColorSpace_for_spots(ctx, o, shading_dict, page_dict, tracker->spot_dict);
@@ -257,13 +276,16 @@ static int pdfi_check_Shading_dict(pdf_context *ctx, pdf_dict *shading_dict,
     if (resource_is_checked(tracker, (pdf_obj *)shading_dict))
         return 0;
 
+    if (pdfi_type_of(shading_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     if (pdfi_dict_entries(shading_dict) > 0) {
         code = pdfi_loop_detector_mark(ctx); /* Mark the start of the Shading dictionary loop */
         if (code < 0)
             return code;
 
         code = pdfi_dict_first(ctx, shading_dict, &Key, &Value, &index);
-        if (code < 0 || !(Value->type == PDF_DICT || Value->type == PDF_STREAM))
+        if (code < 0 || !(pdfi_type_of(Value) == PDF_DICT || pdfi_type_of(Value) == PDF_STREAM))
             goto error1;
 
         i = 1;
@@ -290,7 +312,7 @@ static int pdfi_check_Shading_dict(pdf_context *ctx, pdf_dict *shading_dict,
                 }
 
                 code = pdfi_dict_next(ctx, shading_dict, &Key, &Value, &index);
-                if (code == 0 && Value->type == PDF_DICT)
+                if (code == 0 && pdfi_type_of(Value) == PDF_DICT)
                     break;
                 pdfi_countdown(Key);
                 Key = NULL;
@@ -326,6 +348,9 @@ static int pdfi_check_XObject(pdf_context *ctx, pdf_dict *xobject, pdf_dict *pag
     if (resource_is_checked(tracker, (pdf_obj *)xobject))
         return 0;
 
+    if (pdfi_type_of(xobject) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     code = pdfi_dict_get_type(ctx, xobject, "Subtype", PDF_NAME, (pdf_obj **)&n);
     if (code >= 0) {
         if (pdfi_name_is((const pdf_name *)n, "Image")) {
@@ -387,6 +412,11 @@ static int pdfi_check_XObject(pdf_context *ctx, pdf_dict *xobject, pdf_dict *pag
 
                 code = pdfi_dict_knownget_type(ctx, xobject, "Resources", PDF_DICT, (pdf_obj **)&resource_dict);
                 if (code > 0) {
+                    if (ctx->loop_detection && pdf_object_num((pdf_obj *)resource_dict) != 0) {
+                        code = pdfi_loop_detector_add_object(ctx, resource_dict->object_num);
+                        if (code < 0)
+                            goto transparency_exit;
+                    }
                     code = pdfi_check_Resources(ctx, resource_dict, page_dict, tracker);
                     pdfi_countdown(resource_dict);
                     if (code < 0)
@@ -417,6 +447,9 @@ static int pdfi_check_XObject_dict(pdf_context *ctx, pdf_dict *xobject_dict, pdf
     if (resource_is_checked(tracker, (pdf_obj *)xobject_dict))
         return 0;
 
+    if (pdfi_type_of(xobject_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     if (pdfi_dict_entries(xobject_dict) > 0) {
         code = pdfi_loop_detector_mark(ctx); /* Mark the start of the XObject dictionary loop */
         if (code < 0)
@@ -425,7 +458,7 @@ static int pdfi_check_XObject_dict(pdf_context *ctx, pdf_dict *xobject_dict, pdf
         code = pdfi_dict_first(ctx, xobject_dict, &Key, &Value, &index);
         if (code < 0)
             goto error_exit;
-        if (Value->type != PDF_STREAM)
+        if (pdfi_type_of(Value) != PDF_STREAM)
             goto error_exit;
 
         i = 1;
@@ -457,7 +490,7 @@ static int pdfi_check_XObject_dict(pdf_context *ctx, pdf_dict *xobject_dict, pdf
                 }
 
                 code = pdfi_dict_next(ctx, xobject_dict, &Key, &Value, &index);
-                if (code == 0 && Value->type == PDF_STREAM)
+                if (code == 0 && pdfi_type_of(Value) == PDF_STREAM)
                     break;
                 pdfi_countdown(Key);
                 Key = NULL;
@@ -490,6 +523,9 @@ static int pdfi_check_ExtGState(pdf_context *ctx, pdf_dict *extgstate_dict, pdf_
     if (resource_is_checked(tracker, (pdf_obj *)extgstate_dict))
         return 0;
 
+    if (pdfi_type_of(extgstate_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     if (pdfi_dict_entries(extgstate_dict) > 0) {
         /* See if /OP or /op is true */
         code = pdfi_dict_get_bool(ctx, extgstate_dict, "OP", &overprint);
@@ -502,14 +538,16 @@ static int pdfi_check_ExtGState(pdf_context *ctx, pdf_dict *extgstate_dict, pdf_
         /* Check SMask */
         code = pdfi_dict_knownget(ctx, extgstate_dict, "SMask", &o);
         if (code > 0) {
-            if (o->type == PDF_NAME) {
-                if (!pdfi_name_is((pdf_name *)o, "None")) {
-                    pdfi_countdown(o);
-                    tracker->transparent = true;
-                    return 0;
-                }
-            } else {
-                if (o->type == PDF_DICT) {
+            switch (pdfi_type_of(o)) {
+                case PDF_NAME:
+                    if (!pdfi_name_is((pdf_name *)o, "None")) {
+                        pdfi_countdown(o);
+                        tracker->transparent = true;
+                        return 0;
+                    }
+                    break;
+                case PDF_DICT:
+                {
                     pdf_obj *G = NULL;
 
                     tracker->transparent = true;
@@ -526,6 +564,8 @@ static int pdfi_check_ExtGState(pdf_context *ctx, pdf_dict *extgstate_dict, pdf_
                     pdfi_countdown(o);
                     return code;
                 }
+                default:
+                    break;
             }
         }
         pdfi_countdown(o);
@@ -577,6 +617,9 @@ static int pdfi_check_ExtGState_dict(pdf_context *ctx, pdf_dict *extgstate_dict,
     if (resource_is_checked(tracker, (pdf_obj *)extgstate_dict))
         return 0;
 
+    if (pdfi_type_of(extgstate_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     if (pdfi_dict_entries(extgstate_dict) > 0) {
         code = pdfi_loop_detector_mark(ctx); /* Mark the start of the ColorSpace dictionary loop */
         if (code < 0)
@@ -611,7 +654,7 @@ static int pdfi_check_ExtGState_dict(pdf_context *ctx, pdf_dict *extgstate_dict,
                 }
 
                 code = pdfi_dict_next(ctx, extgstate_dict, &Key, &Value, &index);
-                if (code == 0 && Value->type == PDF_DICT)
+                if (code == 0 && pdfi_type_of(Value) == PDF_DICT)
                     break;
                 pdfi_countdown(Key);
                 Key = NULL;
@@ -644,6 +687,9 @@ static int pdfi_check_Pattern(pdf_context *ctx, pdf_dict *pattern, pdf_dict *pag
     if (resource_is_checked(tracker, (pdf_obj *)pattern))
         return 0;
 
+    if (pdfi_type_of(pattern) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     if (tracker->spot_dict != NULL) {
         code = pdfi_dict_knownget(ctx, pattern, "Shading", &o);
         if (code > 0)
@@ -677,7 +723,7 @@ int pdfi_check_Pattern_transparency(pdf_context *ctx, pdf_dict *pattern, pdf_dic
                                     bool *transparent)
 {
     int code;
-    pdfi_check_tracker_t tracker = {0, 0, NULL, 0, NULL};
+    pdfi_check_tracker_t tracker = {0, 0, NULL, NULL, 0, NULL};
 
     /* NOTE: We use a "null" tracker that won't do any optimization to prevent
      * checking the same resource twice.
@@ -708,6 +754,9 @@ static int pdfi_check_Pattern_dict(pdf_context *ctx, pdf_dict *pattern_dict, pdf
     if (resource_is_checked(tracker, (pdf_obj *)pattern_dict))
         return 0;
 
+    if (pdfi_type_of(pattern_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     if (pdfi_dict_entries(pattern_dict) > 0) {
         code = pdfi_loop_detector_mark(ctx); /* Mark the start of the Pattern dictionary loop */
         if (code < 0)
@@ -717,7 +766,7 @@ static int pdfi_check_Pattern_dict(pdf_context *ctx, pdf_dict *pattern_dict, pdf
         if (code < 0)
             goto error1;
 
-        if(Value->type != PDF_DICT && Value->type != PDF_STREAM)
+        if (pdfi_type_of(Value) != PDF_DICT && pdfi_type_of(Value) != PDF_STREAM)
             goto transparency_exit;
 
         i = 1;
@@ -748,7 +797,7 @@ static int pdfi_check_Pattern_dict(pdf_context *ctx, pdf_dict *pattern_dict, pdf
                 }
 
                 code = pdfi_dict_next(ctx, pattern_dict, &Key, &Value, &index);
-                if (code == 0 && (Value->type == PDF_DICT || Value->type == PDF_STREAM))
+                if (code == 0 && (pdfi_type_of(Value) == PDF_DICT || pdfi_type_of(Value) == PDF_STREAM))
                     break;
                 pdfi_countdown(Key);
                 Key = NULL;
@@ -771,7 +820,8 @@ error1:
 
 /*
  * This routine checks a Font dictionary to see if it contains any spot
- * colour definitions, or transparency usage.
+ * colour definitions, or transparency usage. While we are here, if the tracker's font_array
+ * is not NULL, pick up the font information and store it in the array.
  */
 static int pdfi_check_Font(pdf_context *ctx, pdf_dict *font, pdf_dict *page_dict,
                            pdfi_check_tracker_t *tracker)
@@ -782,22 +832,186 @@ static int pdfi_check_Font(pdf_context *ctx, pdf_dict *font, pdf_dict *page_dict
     if (resource_is_checked(tracker, (pdf_obj *)font))
         return 0;
 
-    if (font->type != PDF_DICT)
+    if (pdfi_type_of(font) != PDF_DICT)
         return_error(gs_error_typecheck);
 
-    code = pdfi_dict_knownget_type(ctx, font, "Subtype", PDF_NAME, &o);
-    if (code > 0) {
-        if (pdfi_name_is((pdf_name *)o, "Type3")) {
-            pdfi_countdown(o);
-            o = NULL;
+    if (tracker->font_array != NULL) {
+        /* If we get to here this is a font we have not seen before. We need
+         * to make a new font array big enough to hold the existing entries +1
+         * copy the existing entries to the new array and free the old array.
+         * Finally create a dictionary with all the font information we want
+         * and add it to the array.
+         */
+        pdf_array *new_fonts = NULL;
+        int index = 0;
+        pdf_obj *array_obj = NULL;
+        pdf_dict *font_info_dict = NULL;
 
-            code = pdfi_dict_knownget_type(ctx, font, "Resources", PDF_DICT, &o);
-            if (code > 0)
-                (void)pdfi_check_Resources(ctx, (pdf_dict *)o, page_dict, tracker);
+        /* Let's start by gathering the information we need and storing it in a dictionary */
+        code = pdfi_dict_alloc(ctx, 4, &font_info_dict);
+        if (code < 0)
+            return code;
+        pdfi_countup(font_info_dict);
+
+        if (font->object_num != 0) {
+            pdf_num *int_obj = NULL;
+
+            code = pdfi_object_alloc(ctx, PDF_INT, 0, (pdf_obj **)&int_obj);
+            if (code >= 0) {
+                pdfi_countup(int_obj);
+                int_obj->value.i = font->object_num;
+                code = pdfi_dict_put(ctx, font_info_dict, "ObjectNum", (pdf_obj *)int_obj);
+                pdfi_countdown(int_obj);
+            }
+            if (code < 0) {
+                pdfi_countdown(font_info_dict);
+                return code;
+            }
         }
+
+        code = pdfi_dict_get(ctx, font, "BaseFont", &array_obj);
+        if (code >= 0) {
+            code = pdfi_dict_put(ctx, font_info_dict, "BaseFont", array_obj);
+            if (code < 0) {
+                pdfi_countdown(array_obj);
+                pdfi_countdown(font_info_dict);
+                return code;
+            }
+        }
+        pdfi_countdown(array_obj);
+        array_obj = NULL;
+
+        code = pdfi_dict_get(ctx, font, "ToUnicode", &array_obj);
+        if (code >= 0)
+            code = pdfi_dict_put(ctx, font_info_dict, "ToUnicode", PDF_TRUE_OBJ);
+        else
+            code = pdfi_dict_put(ctx, font_info_dict, "ToUnicode", PDF_FALSE_OBJ);
+        pdfi_countdown(array_obj);
+        array_obj = NULL;
+        if (code < 0)
+            return code;
+
+        code = pdfi_dict_get(ctx, font, "FontDescriptor", &array_obj);
+        if (code >= 0) {
+            bool known = false;
+
+            (void)pdfi_dict_known(ctx, (pdf_dict *)array_obj, "FontFile", &known);
+            if (!known) {
+                (void)pdfi_dict_known(ctx, (pdf_dict *)array_obj, "FontFile2", &known);
+                if (!known) {
+                    (void)pdfi_dict_known(ctx, (pdf_dict *)array_obj, "FontFile3", &known);
+                }
+            }
+
+            if (known >= 0)
+                code = pdfi_dict_put(ctx, font_info_dict, "Embedded", PDF_TRUE_OBJ);
+            else
+                code = pdfi_dict_put(ctx, font_info_dict, "Embedded", PDF_FALSE_OBJ);
+        } else
+            code = pdfi_dict_put(ctx, font_info_dict, "Embedded", PDF_FALSE_OBJ);
+
+        pdfi_countdown(array_obj);
+        array_obj = NULL;
+
+        if (code < 0)
+            return code;
+
+
+        code = pdfi_dict_knownget_type(ctx, font, "Subtype", PDF_NAME, &array_obj);
+        if (code > 0) {
+            code = pdfi_dict_put(ctx, font_info_dict, "Subtype", array_obj);
+            if (code < 0) {
+                pdfi_countdown(array_obj);
+                pdfi_countdown(font_info_dict);
+                return code;
+            }
+
+            if (pdfi_name_is((pdf_name *)array_obj, "Type3")) {
+                pdfi_countdown(o);
+                o = NULL;
+
+                code = pdfi_dict_knownget_type(ctx, font, "Resources", PDF_DICT, &o);
+                if (code > 0)
+                    (void)pdfi_check_Resources(ctx, (pdf_dict *)o, page_dict, tracker);
+            }
+
+            if (pdfi_name_is((const pdf_name *)array_obj, "Type0")){
+                pdf_array *descendants = NULL;
+                pdf_dict *desc_font = NULL;
+
+                code = pdfi_dict_get(ctx, font, "DescendantFonts", (pdf_obj **)&descendants);
+                if (code >= 0) {
+                    code = pdfi_array_get(ctx, descendants, 0, (pdf_obj **)&desc_font);
+                    if (code >= 0){
+                        pdf_array *desc_array = NULL;
+
+                        code = pdfi_array_alloc(ctx, 0, &desc_array);
+                        pdfi_countup(desc_array);
+                        if (code >= 0) {
+                            pdf_array *saved = tracker->font_array;
+
+                            tracker->font_array = desc_array;
+                            (void)pdfi_check_Font(ctx, desc_font, page_dict, tracker);
+                            (void)pdfi_dict_put(ctx, font_info_dict, "Descendants", (pdf_obj *)tracker->font_array);
+                            pdfi_countdown((pdf_obj *)tracker->font_array);
+                            tracker->font_array = saved;
+                        }
+                        pdfi_countdown(descendants);
+                        pdfi_countdown(desc_font);
+                    }
+                }
+            }
+        }
+        pdfi_countdown(array_obj);
+        array_obj = NULL;
+
+        code = pdfi_array_alloc(ctx, pdfi_array_size(tracker->font_array) + 1, &new_fonts);
+        if (code < 0) {
+            pdfi_countdown(font_info_dict);
+            return code;
+        }
+        pdfi_countup(new_fonts);
+
+        for (index = 0; index < pdfi_array_size(tracker->font_array); index++) {
+            code = pdfi_array_get(ctx, tracker->font_array, index, &array_obj);
+            if (code < 0) {
+                pdfi_countdown(font_info_dict);
+                pdfi_countdown(new_fonts);
+                return code;
+            }
+            code = pdfi_array_put(ctx, new_fonts, index, array_obj);
+            pdfi_countdown(array_obj);
+            if (code < 0) {
+                pdfi_countdown(font_info_dict);
+                pdfi_countdown(new_fonts);
+                return code;
+            }
+        }
+        code = pdfi_array_put(ctx, new_fonts, index, (pdf_obj *)font_info_dict);
+        if (code < 0) {
+            pdfi_countdown(font_info_dict);
+            pdfi_countdown(new_fonts);
+            return code;
+        }
+        pdfi_countdown(font_info_dict);
+        pdfi_countdown(tracker->font_array);
+        tracker->font_array = new_fonts;
+    } else {
+        code = pdfi_dict_knownget_type(ctx, font, "Subtype", PDF_NAME, &o);
+        if (code > 0) {
+            if (pdfi_name_is((pdf_name *)o, "Type3")) {
+                pdfi_countdown(o);
+                o = NULL;
+
+                code = pdfi_dict_knownget_type(ctx, font, "Resources", PDF_DICT, &o);
+                if (code > 0)
+                    (void)pdfi_check_Resources(ctx, (pdf_dict *)o, page_dict, tracker);
+            }
+        }
+
+        pdfi_countdown(o);
+        o = NULL;
     }
-    pdfi_countdown(o);
-    o = NULL;
 
     return 0;
 }
@@ -808,13 +1022,16 @@ static int pdfi_check_Font(pdf_context *ctx, pdf_dict *font, pdf_dict *page_dict
 static int pdfi_check_Font_dict(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *page_dict,
                                 pdfi_check_tracker_t *tracker)
 {
-    int code;
+    int code = 0;
     uint64_t i, index;
     pdf_obj *Key = NULL, *Value = NULL;
 
     if (resource_is_checked(tracker, (pdf_obj *)font_dict))
         return 0;
 
+    if (pdfi_type_of(font_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     if (pdfi_dict_entries(font_dict) > 0) {
         code = pdfi_loop_detector_mark(ctx); /* Mark the start of the Font dictionary loop */
         if (code < 0)
@@ -826,7 +1043,17 @@ static int pdfi_check_Font_dict(pdf_context *ctx, pdf_dict *font_dict, pdf_dict
 
         i = 1;
         do {
-            code = pdfi_check_Font(ctx, (pdf_dict *)Value, page_dict, tracker);
+            if (pdfi_type_of(Value) == PDF_DICT)
+                code = pdfi_check_Font(ctx, (pdf_dict *)Value, page_dict, tracker);
+            else if (pdfi_type_of(Value) == PDF_FONT) {
+                pdf_dict *d = ((pdf_font *)Value)->PDF_font;
+
+                code = pdfi_check_Font(ctx, d, page_dict, tracker);
+            } else {
+                pdfi_set_warning(ctx, 0, NULL, W_PDF_FONTRESOURCE_TYPE, "pdfi_check_Font_dict", "");
+            }
+            if (code < 0)
+                break;
 
             pdfi_countdown(Key);
             Key = NULL;
@@ -839,30 +1066,23 @@ static int pdfi_check_Font_dict(pdf_context *ctx, pdf_dict *font_dict, pdf_dict
             if (code < 0)
                 goto error1;
 
-            do {
-                if (i++ >= pdfi_dict_entries(font_dict)) {
-                    code = 0;
-                    goto transparency_exit;
-                }
+            if (i++ >= pdfi_dict_entries(font_dict)) {
+                code = 0;
+                break;
+            }
+
+            code = pdfi_dict_next(ctx, font_dict, &Key, &Value, &index);
+            if (code < 0)
+                break;
 
-                code = pdfi_dict_next(ctx, font_dict, &Key, &Value, &index);
-                if (code == 0 && Value->type == PDF_DICT)
-                    break;
-                pdfi_countdown(Key);
-                Key = NULL;
-                pdfi_countdown(Value);
-                Value = NULL;
-            } while(1);
         }while (1);
     }
-    return 0;
 
-transparency_exit:
+    (void)pdfi_loop_detector_cleartomark(ctx); /* Clear to the mark for the current resource loop */
     pdfi_countdown(Key);
     pdfi_countdown(Value);
 
 error1:
-    (void)pdfi_loop_detector_cleartomark(ctx); /* Clear to the mark for the current resource loop */
     return code;
 }
 
@@ -875,12 +1095,17 @@ static int pdfi_check_Resources(pdf_context *ctx, pdf_dict *Resources_dict,
     if (resource_is_checked(tracker, (pdf_obj *)Resources_dict))
         return 0;
 
+    if (pdfi_type_of(Resources_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     /* First up, check any colour spaces, for new spot colours.
      * We only do this if asked because its expensive. spot_dict being NULL
      * means we aren't interested in spot colours (not a DeviceN or Separation device)
      */
     if (tracker->spot_dict != NULL) {
         code = pdfi_dict_knownget_type(ctx, Resources_dict, "ColorSpace", PDF_DICT, &d);
+        if (code == gs_error_typecheck)
+            pdfi_set_error(ctx, 0, NULL, E_PDF_SPOT_CHK_BADTYPE, "pdfi_check_Resources", "");
         if (code > 0)
             (void)pdfi_check_ColorSpace_dict(ctx, (pdf_dict *)d, page_dict, tracker);
 
@@ -888,6 +1113,8 @@ static int pdfi_check_Resources(pdf_context *ctx, pdf_dict *Resources_dict,
         d = NULL;
 
         code = pdfi_dict_knownget_type(ctx, Resources_dict, "Shading", PDF_DICT, &d);
+        if (code == gs_error_typecheck)
+            pdfi_set_error(ctx, 0, NULL, E_PDF_SPOT_CHK_BADTYPE, "pdfi_check_Resources", "");
         if (code > 0)
             (void)pdfi_check_Shading_dict(ctx, (pdf_dict *)d, page_dict, tracker);
         pdfi_countdown(d);
@@ -895,18 +1122,24 @@ static int pdfi_check_Resources(pdf_context *ctx, pdf_dict *Resources_dict,
     }
 
     code = pdfi_dict_knownget_type(ctx, Resources_dict, "XObject", PDF_DICT, &d);
+    if (code == gs_error_typecheck)
+        pdfi_set_error(ctx, 0, NULL, E_PDF_TRANS_CHK_BADTYPE, "pdfi_check_Resources", "");
     if (code > 0)
         (void)pdfi_check_XObject_dict(ctx, (pdf_dict *)d, page_dict, tracker);
     pdfi_countdown(d);
     d = NULL;
 
     code = pdfi_dict_knownget_type(ctx, Resources_dict, "Pattern", PDF_DICT, &d);
+    if (code == gs_error_typecheck)
+        pdfi_set_error(ctx, 0, NULL, E_PDF_TRANS_CHK_BADTYPE, "pdfi_check_Resources", "");
     if (code > 0)
         (void)pdfi_check_Pattern_dict(ctx, (pdf_dict *)d, page_dict, tracker);
     pdfi_countdown(d);
     d = NULL;
 
     code = pdfi_dict_knownget_type(ctx, Resources_dict, "Font", PDF_DICT, &d);
+    if (code == gs_error_typecheck)
+        pdfi_set_error(ctx, 0, NULL, E_PDF_TRANS_CHK_BADTYPE, "pdfi_check_Resources", "");
     if (code > 0)
         (void)pdfi_check_Font_dict(ctx, (pdf_dict *)d, page_dict, tracker);
     /* From this point onwards, if we detect transparency (or have already detected it) we
@@ -916,6 +1149,8 @@ static int pdfi_check_Resources(pdf_context *ctx, pdf_dict *Resources_dict,
     d = NULL;
 
     code = pdfi_dict_knownget_type(ctx, Resources_dict, "ExtGState", PDF_DICT, &d);
+    if (code == gs_error_typecheck)
+        pdfi_set_error(ctx, 0, NULL, E_PDF_TRANS_CHK_BADTYPE, "pdfi_check_Resources", "");
     if (code > 0)
         (void)pdfi_check_ExtGState_dict(ctx, (pdf_dict *)d, page_dict, tracker);
     pdfi_countdown(d);
@@ -937,6 +1172,9 @@ static int pdfi_check_annot_for_transparency(pdf_context *ctx, pdf_dict *annot,
     if (resource_is_checked(tracker, (pdf_obj *)annot))
         return 0;
 
+    if (pdfi_type_of(annot) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
     /* Check #1 Does the (Normal) Appearnce stream use any Resources which include transparency.
      * We check this first, because this also checks for spot colour spaces. Once we've done that we
      * can exit the checks as soon as we detect transparency.
@@ -1035,6 +1273,9 @@ static int pdfi_check_Annots_for_transparency(pdf_context *ctx, pdf_array *annot
     if (resource_is_checked(tracker, (pdf_obj *)annots_array))
         return 0;
 
+    if (pdfi_type_of(annots_array) != PDF_ARRAY)
+        return_error(gs_error_typecheck);
+
     for (i=0; i < pdfi_array_size(annots_array); i++) {
         code = pdfi_array_get_type(ctx, annots_array, (uint64_t)i, PDF_DICT, (pdf_obj **)&annot);
         if (code >= 0) {
@@ -1084,6 +1325,10 @@ static int pdfi_check_page_inner(pdf_context *ctx, pdf_dict *page_dict,
 
     tracker->transparent = false;
 
+    if (pdfi_type_of(page_dict) != PDF_DICT)
+        return_error(gs_error_typecheck);
+
+
     /* Check if the page dictionary has a page Group entry (for spots).
      * Page group should mean the page has transparency but we ignore it for the purposes
      * of transparency detection. See above.
@@ -1104,7 +1349,7 @@ static int pdfi_check_page_inner(pdf_context *ctx, pdf_dict *page_dict,
     code = pdfi_dict_knownget_type(ctx, page_dict, "Resources", PDF_DICT, (pdf_obj **)&Resources);
     if (code > 0)
         code = pdfi_check_Resources(ctx, Resources, page_dict, tracker);
-    if ((code < 0 && ctx->args.pdfstoponerror) || (code == gs_error_stackoverflow))
+    if ((code < 0 && ctx->args.pdfstoponerror) || (code == gs_error_pdf_stackoverflow))
         goto exit;
 
     /* If we are drawing Annotations, check to see if the page uses any Annots */
@@ -1130,7 +1375,7 @@ static int pdfi_check_page_inner(pdf_context *ctx, pdf_dict *page_dict,
  * Sets ctx->page.has_transparency and ctx->page.num_spots
  * do_setup -- indicates whether to actually set up the device with the spot count.
  */
-int pdfi_check_page(pdf_context *ctx, pdf_dict *page_dict, bool do_setup)
+int pdfi_check_page(pdf_context *ctx, pdf_dict *page_dict, pdf_array **fonts_array, pdf_array **spots_array, bool do_setup)
 {
     int code;
     int spots = 0;
@@ -1139,13 +1384,7 @@ int pdfi_check_page(pdf_context *ctx, pdf_dict *page_dict, bool do_setup)
     ctx->page.num_spots = 0;
     ctx->page.has_transparency = false;
 
-    /* Need to do this here so that pdfi_check_init_tracker will be setup
-     * for spot colours, if the device is spot colour capable.
-     * It is also called in pdfi_page_render() and pdfi_process_pdf_file()
-     * TODO: Should probably look into that..
-     */
-    pdfi_device_set_flags(ctx);
-    code = pdfi_check_init_tracker(ctx, &tracker);
+    code = pdfi_check_init_tracker(ctx, &tracker, fonts_array, spots_array);
     if (code < 0)
         goto exit;
 
@@ -1191,7 +1430,7 @@ int pdfi_check_page(pdf_context *ctx, pdf_dict *page_dict, bool do_setup)
                     code = pdfi_dict_first(ctx, tracker.spot_dict, (pdf_obj **)&Key, &Value, &index);
                     while (code >= 0)
                     {
-                        if (Key->type == PDF_NAME) {
+                        if (pdfi_type_of(Key) == PDF_NAME) {
                             table[a].data = ((pdf_string *)Key)->data;
                             table[a].size = ((pdf_string *)Key)->length;
                             table[a++].persistent = false;
@@ -1263,6 +1502,47 @@ int pdfi_check_page(pdf_context *ctx, pdf_dict *page_dict, bool do_setup)
         ctx->page.has_OP = false;
 
  exit:
+    if (fonts_array != NULL) {
+        *fonts_array = tracker.font_array;
+        pdfi_countup(*fonts_array);
+    }
+
+    if (spots_array != NULL && tracker.spot_dict != NULL && pdfi_dict_entries(tracker.spot_dict) != 0) {
+        pdf_array *new_array = NULL;
+        pdf_name *Key = NULL;
+        pdf_obj *Value = NULL;
+        uint64_t index = 0, a_index = 0;
+
+        index = pdfi_dict_entries(tracker.spot_dict);
+
+        code = pdfi_array_alloc(ctx, index, &new_array);
+        if (code < 0)
+            goto error;
+        pdfi_countup(new_array);
+
+        code = pdfi_dict_first(ctx, tracker.spot_dict, (pdf_obj **)&Key, &Value, &index);
+        while (code >= 0)
+        {
+            if (pdfi_type_of(Key) == PDF_NAME) {
+                code = pdfi_array_put(ctx, new_array, a_index++, (pdf_obj *)Key);
+                if (code < 0) {
+                    pdfi_countdown(new_array);
+                    pdfi_countdown(Key);
+                    pdfi_countdown(Value);
+                    goto error;
+                }
+            }
+
+            pdfi_countdown(Key);
+            Key = NULL;
+            pdfi_countdown(Value);
+            Value = NULL;
+            code = pdfi_dict_next(ctx, tracker.spot_dict, (pdf_obj **)&Key, &Value, &index);
+        }
+        code = 0;
+        *spots_array = new_array;
+    }
+error:
     (void)pdfi_check_free_tracker(ctx, &tracker);
     return code;
 }
